#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

int random_dec(int current)
{
    int retVal = current - random()%10;
    return (retVal > 0) ? retVal : 0;
}

int main(int argc, char **argv)
{
    /* MPI Initialization */
    MPI_Init(&argc, &argv);

    int myRank, numProcs;

    MPI_Comm_size(MPI_COMM_WORLD, &numProcs);
    MPI_Comm_rank(MPI_COMM_WORLD, &myRank);

    /* Rank-specific initialization of the random number generator */
    srandom(MPI_Wtime() + myRank*100);

    int nextRank = (myRank + 1) % numProcs;
    int looserRank = -1;

    int countDownCounter = 50;

    /* Rank 0 starts the bomb */
    if (myRank == 0)
    {
	if (argc > 1)
	{
	    countDownCounter = atoi(argv[1]);
	    if (countDownCounter <= 0) exit(1);
	}
	printf("Counting down from %i\n", countDownCounter);
	MPI_Send(&countDownCounter, 1, MPI_INT, nextRank, 1, MPI_COMM_WORLD);
    }

    char myString[MPI_MAX_PROCESSOR_NAME];
    int myStringLength;

    MPI_Get_processor_name(myString, &myStringLength);
    printf("I am rank %i running on %s\n", myRank, myString);	

    int done = 0;

    while (!done)
    {
	MPI_Status status;
	int data;

	MPI_Recv(&data, 1, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG,
		 MPI_COMM_WORLD, &status);
	if (status.MPI_TAG == 2)
	{
	    /* The shutdown message was received*/ 
	    done = 1;
	    looserRank = data;			
	}
	else
	{
	    countDownCounter = random_dec(data);
	    if (countDownCounter > 0)
	    {
		printf("Process %i has received the bomb (%i on the clock)"
		       " and is still alive!\n", myRank, countDownCounter);
		MPI_Send(&countDownCounter, 1, MPI_INT, nextRank, 1,
			 MPI_COMM_WORLD);
	    }
	    else
	    {
		int i;

		printf("Process %i lost\n", myRank);
		for (i = 0; i < numProcs; i++)
		{
		    MPI_Request request;

		    MPI_Isend(&myRank, 1, MPI_INT, i, 2,
			      MPI_COMM_WORLD, &request);
		    MPI_Request_free(&request);
		}
	    }
	}
    }
    printf("I am process %i and %i is the looser\n", myRank, looserRank);

    MPI_Finalize();
    return 0;
}
